﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Praktikum_3_Loesung
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        Fahrzeug[] fahrzeuge = new Fahrzeug[]
        {
            new Fahrzeug("AB 123 XY", 1.6, 200.0),
            new Fahrzeug("CD 456 UV", 2.1, 300.0),
            new Fahrzeug("EF 789 PQ", 3.6, 400.0),
        };

        private void buttonHinzufügen_Click(object sender, RoutedEventArgs e)
        {
            string[] start = textBoxStartzeit.Text.Split(':');
            string[] end = textBoxEndzeit.Text.Split(':');
            DateTime morgen = (DateTime.Now.Date + TimeSpan.FromHours(30)).Date;
            Zeitraum z = new Zeitraum(morgen + TimeSpan.FromHours(int.Parse(start[0])) + TimeSpan.FromMinutes(int.Parse(start[1])),
                morgen + TimeSpan.FromHours(int.Parse(end[0])) + TimeSpan.FromMinutes(int.Parse(end[1])));
            Buchung b = null;
            if (checkBoxPriorität.IsChecked == true)
            {
                b = new BuchungMitPriorität(textBoxName.Text, z, double.Parse(textBoxKm.Text));
            }
            else
            {
                b = new Buchung(textBoxName.Text, z, double.Parse(textBoxKm.Text));
            }
            listBoxBuchungen.Items.Add(b);
        }

        static Random random = new Random();

        private void buttonLösen_Click(object sender, RoutedEventArgs e)
        {
            int[] bisherBesteZuordnungen = null; // enthält später: Buchung -> Index des Fahrzeugs
            double bisherBesterVerlust = double.MaxValue;

            // naive, rein stochastische Optimierung; eine Stufe besser wäre ein evolutionärer Algorithmus
            for (int k = 0; k < 100000; k++)
            {
                int[] zuordnungen = new int[listBoxBuchungen.Items.Count];
                for (int i = 0; i < zuordnungen.Length; i++)
                {
                    zuordnungen[i] = random.Next(fahrzeuge.Length);
                }

                double verlust = 0.0;
                for (int f = 0; f < fahrzeuge.Length; f++)
                {
                    List<Buchung> buchungenFürDiesesFahrzeug = new List<Buchung>();
                    for (int i = 0; i < zuordnungen.Length; i++)
                    {
                        if (zuordnungen[i] == f)
                        {
                            buchungenFürDiesesFahrzeug.Add((Buchung)listBoxBuchungen.Items[i]);
                        }
                    }

                    Buchung[] sortiert = buchungenFürDiesesFahrzeug.OrderBy(b => b.Buchungszeitraum.Von).ToArray();

                    double aktuelleReichweite = fahrzeuge[f].ReichweiteInKmBeiVollladung;
                    for (int i = 0; i < sortiert.Length; i++)
                    {
                        Buchung bi = sortiert[i];
                        if(bi.StreckeInKm > aktuelleReichweite)
                        {
                            verlust += (bi.StreckeInKm - aktuelleReichweite) * (bi is BuchungMitPriorität ? 10000.0 : 1000.0);
                        }

                        aktuelleReichweite -= bi.StreckeInKm;
                        if(aktuelleReichweite < 0.0)
                        {
                            aktuelleReichweite = 0.0;
                        }

                        if (i < sortiert.Length - 1)
                        {
                            Buchung bNächste = sortiert[i + 1];
                            double ladezeitInMinuten = (bNächste.Buchungszeitraum.Von - bi.Buchungszeitraum.Bis).TotalMinutes;
                            if(ladezeitInMinuten >= 0.0)
                            {
                                aktuelleReichweite += ladezeitInMinuten * fahrzeuge[f].ReichweiteInKmBeiVollladung / (fahrzeuge[f].LadedauerBisVollladungInStunden * 60.0);
                                if(aktuelleReichweite > fahrzeuge[f].ReichweiteInKmBeiVollladung)
                                {
                                    aktuelleReichweite = fahrzeuge[f].ReichweiteInKmBeiVollladung;
                                }
                            }
                            else // Überlapp
                            {
                                verlust += Math.Abs(ladezeitInMinuten) * (bi is BuchungMitPriorität ? 10.0 : 1.0);
                            }
                        }
                    }
                }

                if(verlust < bisherBesterVerlust)
                {
                    bisherBesterVerlust = verlust;
                    bisherBesteZuordnungen = zuordnungen;
                }
            }

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < bisherBesteZuordnungen.Length; i++)
            {
                sb.AppendLine(listBoxBuchungen.Items[i] + ": " + fahrzeuge[bisherBesteZuordnungen[i]]);
            }
            sb.AppendLine("Verlust = " + bisherBesterVerlust);
            MessageBox.Show(sb.ToString());
        }
    }
}
